--- crypto_drv.c.orig	Wed Oct 30 10:53:31 2002
+++ crypto_drv.c	Wed Oct 30 01:11:05 2002
@@ -33,6 +33,10 @@
 #include "md5.h"
 #include "sha.h"
 
+#if defined(HAVE_MACH_O_DYLD_H)
+#include <mach-o/dyld.h>
+#endif
+
 #define get_int32(s) ((((unsigned char*) (s))[0] << 24) | \
                       (((unsigned char*) (s))[1] << 16) | \
                       (((unsigned char*) (s))[2] << 8)  | \
@@ -40,10 +44,15 @@
 
 /* shared libs & dlls */
 EXTERN void *driver_dl_open(char *);
-EXTERN void *driver_dl_sym(void *, char *);
 EXTERN int driver_dl_close(void *);
 EXTERN char *driver_dl_error(void);
 
+#if defined(HAVE_MACH_O_DYLD_H)
+void *driver_dl_sym(void *, char *);
+#else
+EXTERN void *driver_dl_sym(void *, char *);
+#endif
+
 /* Driver interface declarations */
 static ErlDrvData start(ErlDrvPort port, char *command);
 static void stop(ErlDrvData drv_data);
@@ -124,6 +133,21 @@
 } crypto_funcs;
 
 static crypto_funcs cfs;
+
+/* We need our own OS X driver_dl_sym because the mechanism used in unix_ddll_drv.c
+   cannot find symbols except those specifically defined IN the module - not those in a
+   linked library */
+
+#if defined(HAVE_MACH_O_DYLD_H)
+void *driver_dl_sym(void *handle, char *symbol) {
+  NSSymbol nssymbol;
+  static char undersym[65]; /* enough for all the symbols we will find here anyway */
+  int sym_len = strlen(symbol);
+  snprintf(undersym,64,"_%s",symbol);
+  nssymbol = NSLookupAndBindSymbol(undersym);
+  return nssymbol != NULL ? NSAddressOfSymbol(nssymbol) : NULL;
+}
+#endif
 
 /* INITIALIZATION AFTER LOADING */
 
